/* $Id: tf_open.c,v 1.1.1.1 2005/03/22 15:43:26 purbanec Exp $ */

/*

  Copyright (c) 2005 Steve Bennett <msteveb at ozemail.com.au>

  This file is part of libtopfield.

  libtopfield is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  puppy is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with puppy; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <stdio.h>
#include <stdlib.h>

#include "usb_io.h"
#include "tf_open.h"
#include "tf_io.h"
#include "usbutil.h"

#define TOPFIELD_VENDOR_ID 0x11DB
#define TOPFIELD_5000PVRT_ID 0x1000

int topfield_open(tf_handle *tf, int index)
{
	tf->dev = open_usb_dev(TOPFIELD_VENDOR_ID, TOPFIELD_5000PVRT_ID, index);

    if (!tf->dev) {
        fprintf(stderr, "Could not find a Topfield TF5000PVRt at index %d\n", index);
        return -1;
    }
	if (tf->timeout == 0) {
		tf->timeout = TF_DEFAULT_TIMEOUT;
	}
	if (tf->tracefh == 0) {
		tf->tracefh = stderr;
	}
	tf->error = 0;
	/* Allocate a buffer big enough for a send/reply packet */
	tf->buf = malloc(0x10000);

	/* Send a cancel in case anything is outstanding */
	if (tf_cmd_cancel(tf) == 0) {
		return 0;
	}

	/* Often the first cancel fails, so try again */
	if (tf_cmd_cancel(tf) == 0) {
		return 0;
	}

	/* If that fails, ... */
	/* First send a fail */
	tf_send_fail(tf, 2);

	/* Now a success */
	tf_send_success(tf);

	/* Try a ready command */
	if (tf_cmd_ready(tf) == 0) {
		return 0;
	}

	/* And finally, one more cancel */
	if (tf_cmd_cancel(tf) == 0) {
		return 0;
	}

	/* Give up */
	topfield_close(tf);
	fprintf(stderr, "Opened Topfield TF5000PVRt at index %d, but could not establish communication\n", index);
	return -1;
}

void topfield_close(tf_handle *tf)
{
	if (tf->dev) {
		close_usb_dev(tf->dev);
		tf->dev = 0;
		free(tf->buf);
		tf->buf = 0;
	}
}
