/* $Id: tf_open.h,v 1.2 2005/04/06 07:03:16 msteveb Exp $ */

/*

  Copyright (c) 2005 Steve Bennett <msteveb at ozemail.com.au>

  This file is part of libtopfield.

  libtopfield is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  puppy is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with puppy; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef TF_OPEN_H
#define TF_OPEN_H

/* Opens a connection to a Topfield 5000PVR device */

#include <stdio.h>

struct usb_dev_handle;

/* Default number of milliseconds to wait for a packet transfer to complete.
 * This needs to be fairly long in case the disk needs to spin up from sleep mode
 */
#define TF_DEFAULT_TIMEOUT 11000

/**
 * This handle represents a connection to a Topfield device.
 */
typedef struct {
	/* The following fields may be set before topfield_open() */
	int timeout;				/* Timeout for commands, in milliseconds */
	int trace_level;			/* 0 = none */
	FILE *tracefh;				/* Debug trace filehandle */

	/* The following fields may be accessed */
	int error;					/* Last error, or 0 if no error */

	/* The following fields should not be touched */
	struct usb_dev_handle *dev;	/* USB handle to the device */
	int pending;				/* A reply should be pending */
	char *buf;					/* Buffer used for transferring data during get/put */
} tf_handle;

/**
 * Opens a connection to the topfield device at the given index
 * (0 is the first, 1 is the seconds, etc.).
 * 
 * '*tf' should be initialised to zeros, except for the fields
 * marked in the tf_handle structure above.
 *
 * Returns 0 if OK, or < 0 on error.
 */
int topfield_open(tf_handle *tf, int index);

/**
 * Closes a previously opened topfield device.
 */
void topfield_close(tf_handle *tf);

#endif
